@@ -28,9 +28,8 @@
 
 #include "../db/commands.h"
 #include "../db/jsobj.h"
-#include "../db/dbmessage.h"
-#include "../db/query.h"
-
+#include "../db/db.h"
+#include "../db/replutil.h"
 #include "../client/connpool.h"
 
 #include "../util/queue.h"
@@ -44,12 +43,13 @@ using namespace std;
 namespace mongo {
 
     // -----ShardingState START ----
-    
+
     ShardingState::ShardingState()
-        : _enabled(false) , _mutex( "ShardingState" ){
+        : _enabled(false) , _mutex( "ShardingState" ),
+          _configServerTickets( 3 /* max number of concurrent config server refresh threads */ ) {
     }
-    
-    void ShardingState::enable( const string& server ){
+
+    void ShardingState::enable( const string& server ) {
         _enabled = true;
         assert( server.size() );
         if ( _configServer.size() == 0 )
@@ -58,55 +58,67 @@ namespace mongo {
             assert( server == _configServer );
         }
     }
-    
-    void ShardingState::gotShardName( const string& name ){
-        if ( _shardName.size() == 0 ){
+
+    void ShardingState::gotShardName( const string& name ) {
+        scoped_lock lk(_mutex);
+        if ( _shardName.size() == 0 ) {
+            // TODO SERVER-2299 verify the name is sound w.r.t IPs
             _shardName = name;
             return;
         }
-        
+
         if ( _shardName == name )
             return;
 
         stringstream ss;
-        ss << "gotShardName different than what i had before " 
-           << " before [" << _shardName << "] " 
-           << " got [" << name << "] " 
-            ;
-        uasserted( 13298 , ss.str() );
+        ss << "gotShardName different than what i had before "
+           << " before [" << _shardName << "] "
+           << " got [" << name << "] "
+           ;
+        msgasserted( 13298 , ss.str() );
     }
-    
-    void ShardingState::gotShardHost( string host ){
-        
+
+    void ShardingState::gotShardHost( string host ) {
+        scoped_lock lk(_mutex);
         size_t slash = host.find( '/' );
         if ( slash != string::npos )
             host = host.substr( 0 , slash );
 
-        if ( _shardHost.size() == 0 ){
+        if ( _shardHost.size() == 0 ) {
             _shardHost = host;
             return;
         }
-        
+
         if ( _shardHost == host )
             return;
 
         stringstream ss;
-        ss << "gotShardHost different than what i had before " 
-           << " before [" << _shardHost << "] " 
-           << " got [" << host << "] " 
-            ;
-        uasserted( 13299 , ss.str() );
+        ss << "gotShardHost different than what i had before "
+           << " before [" << _shardHost << "] "
+           << " got [" << host << "] "
+           ;
+        msgasserted( 13299 , ss.str() );
     }
-    
+
+    void ShardingState::resetShardingState() {
+        scoped_lock lk(_mutex);
+        
+        _enabled = false;
+        _configServer.clear();
+        _shardName.clear();
+        _shardHost.clear();
+        _chunks.clear();
+    }
+
     // TODO we shouldn't need three ways for checking the version. Fix this.
-    bool ShardingState::hasVersion( const string& ns ){
+    bool ShardingState::hasVersion( const string& ns ) {
         scoped_lock lk(_mutex);
 
         ChunkManagersMap::const_iterator it = _chunks.find(ns);
         return it != _chunks.end();
     }
-    
-    bool ShardingState::hasVersion( const string& ns , ConfigVersion& version ){
+
+    bool ShardingState::hasVersion( const string& ns , ConfigVersion& version ) {
         scoped_lock lk(_mutex);
 
         ChunkManagersMap::const_iterator it = _chunks.find(ns);
@@ -117,7 +129,7 @@ namespace mongo {
         version = p->getVersion();
         return true;
     }
-    
+
     const ConfigVersion ShardingState::getVersion( const string& ns ) const {
         scoped_lock lk(_mutex);
 
@@ -125,11 +137,12 @@ namespace mongo {
         if ( it != _chunks.end() ) {
             ShardChunkManagerPtr p = it->second;
             return p->getVersion();
-        } else {
+        }
+        else {
             return 0;
         }
     }
-    
+
     void ShardingState::donateChunk( const string& ns , const BSONObj& min , const BSONObj& max , ShardChunkVersion version ) {
         scoped_lock lk( _mutex );
 
@@ -156,49 +169,70 @@ namespace mongo {
     void ShardingState::splitChunk( const string& ns , const BSONObj& min , const BSONObj& max , const vector<BSONObj>& splitKeys ,
                                     ShardChunkVersion version ) {
         scoped_lock lk( _mutex );
-        
+
         ChunkManagersMap::const_iterator it = _chunks.find( ns );
         assert( it != _chunks.end() ) ;
         ShardChunkManagerPtr p( it->second->cloneSplit( min , max , splitKeys , version ) );
         _chunks[ns] = p;
     }
 
-    void ShardingState::resetVersion( const string& ns ) { 
+    void ShardingState::resetVersion( const string& ns ) {
         scoped_lock lk( _mutex );
 
         _chunks.erase( ns );
     }
-    
+
     bool ShardingState::trySetVersion( const string& ns , ConfigVersion& version /* IN-OUT */ ) {
 
-        // fast path - requested version is at the same version as this chunk manager
+        // Currently this function is called after a getVersion(), which is the first "check", and the assumption here
+        // is that we don't do anything nearly as long as a remote query in a thread between then and now.
+        // Otherwise it may be worth adding an additional check without the _configServerMutex below, since then it
+        // would be likely that the version may have changed in the meantime without waiting for or fetching config results.
+
+        // TODO:  Mutex-per-namespace?
+        
+        LOG( 2 ) << "trying to set shard version of " << version.toString() << " for '" << ns << "'" << endl;
+        
+        _configServerTickets.waitForTicket();
+        TicketHolderReleaser needTicketFrom( &_configServerTickets );
+
+        // fast path - double-check if requested version is at the same version as this chunk manager before verifying
+        // against config server
         //
-        // cases: 
+        // This path will short-circuit the version set if another thread already managed to update the version in the
+        // meantime.  First check is from getVersion().
+        //
+        // cases:
         //   + this shard updated the version for a migrate's commit (FROM side)
         //     a client reloaded chunk state from config and picked the newest version
         //   + two clients reloaded
-        //     one triggered the 'slow path' (below) 
+        //     one triggered the 'slow path' (below)
         //     when the second's request gets here, the version is already current
+        ConfigVersion storedVersion;
         {
             scoped_lock lk( _mutex );
             ChunkManagersMap::const_iterator it = _chunks.find( ns );
-            if ( it != _chunks.end() && it->second->getVersion() == version )
+            if ( it != _chunks.end() && ( storedVersion = it->second->getVersion() ) == version )
                 return true;
         }
+        
+        LOG( 2 ) << "verifying cached version " << storedVersion.toString() << " and new version " << version.toString() << " for '" << ns << "'" << endl;
 
-        // slow path - requested version is different than the current chunk manager's, if one exists, so must check for 
+        // slow path - requested version is different than the current chunk manager's, if one exists, so must check for
         // newest version in the config server
         //
         // cases:
-        //   + a chunk moved TO here 
+        //   + a chunk moved TO here
         //     (we don't bump up the version on the TO side but the commit to config does use higher version)
         //     a client reloads from config an issued the request
         //   + there was a take over from a secondary
         //     the secondary had no state (managers) at all, so every client request will fall here
         //   + a stale client request a version that's not current anymore
 
+        // Can't lock default mutex while creating ShardChunkManager, b/c may have to create a new connection to myself
         const string c = (_configServer == _shardHost) ? "" /* local */ : _configServer;
         ShardChunkManagerPtr p( new ShardChunkManager( c , ns , _shardName ) );
+
         {
             scoped_lock lk( _mutex );
 
@@ -215,7 +249,7 @@ namespace mongo {
         }
     }
 
-    void ShardingState::appendInfo( BSONObjBuilder& b ){
+    void ShardingState::appendInfo( BSONObjBuilder& b ) {
         b.appendBool( "enabled" , _enabled );
         if ( ! _enabled )
             return;
@@ -226,10 +260,10 @@ namespace mongo {
 
         {
             BSONObjBuilder bb( b.subobjStart( "versions" ) );
-            
+
             scoped_lock lk(_mutex);
 
-            for ( ChunkManagersMap::iterator it = _chunks.begin(); it != _chunks.end(); ++it ){
+            for ( ChunkManagersMap::iterator it = _chunks.begin(); it != _chunks.end(); ++it ) {
                 ShardChunkManagerPtr p = it->second;
                 bb.appendTimestamp( it->first , p->getVersion() );
             }
@@ -241,20 +275,21 @@ namespace mongo {
     bool ShardingState::needShardChunkManager( const string& ns ) const {
         if ( ! _enabled )
             return false;
-        
+
         if ( ! ShardedConnectionInfo::get( false ) )
             return false;
 
         return true;
     }
 
-    ShardChunkManagerPtr ShardingState::getShardChunkManager( const string& ns ){
+    ShardChunkManagerPtr ShardingState::getShardChunkManager( const string& ns ) {
         scoped_lock lk( _mutex );
 
         ChunkManagersMap::const_iterator it = _chunks.find( ns );
         if ( it == _chunks.end() ) {
             return ShardChunkManagerPtr();
-        } else {
+        }
+        else {
             return it->second;
         }
     }
@@ -262,27 +297,27 @@ namespace mongo {
     ShardingState shardingState;
 
     // -----ShardingState END ----
-    
+
     // -----ShardedConnectionInfo START ----
 
     boost::thread_specific_ptr<ShardedConnectionInfo> ShardedConnectionInfo::_tl;
 
-    ShardedConnectionInfo::ShardedConnectionInfo(){
+    ShardedConnectionInfo::ShardedConnectionInfo() {
         _forceVersionOk = false;
         _id.clear();
     }
-    
-    ShardedConnectionInfo* ShardedConnectionInfo::get( bool create ){
+
+    ShardedConnectionInfo* ShardedConnectionInfo::get( bool create ) {
         ShardedConnectionInfo* info = _tl.get();
-        if ( ! info && create ){
-            log(1) << "entering shard mode for connection" << endl;
+        if ( ! info && create ) {
+            LOG(1) << "entering shard mode for connection" << endl;
             info = new ShardedConnectionInfo();
             _tl.reset( info );
         }
         return info;
     }
 
-    void ShardedConnectionInfo::reset(){
+    void ShardedConnectionInfo::reset() {
         _tl.reset();
     }
 
@@ -290,41 +325,52 @@ namespace mongo {
         NSVersionMap::const_iterator it = _versions.find( ns );
         if ( it != _versions.end() ) {
             return it->second;
-        } else {
+        }
+        else {
             return 0;
         }
     }
-    
-    void ShardedConnectionInfo::setVersion( const string& ns , const ConfigVersion& version ){
+
+    void ShardedConnectionInfo::setVersion( const string& ns , const ConfigVersion& version ) {
         _versions[ns] = version;
     }
 
-    void ShardedConnectionInfo::setID( const OID& id ){
+    void ShardedConnectionInfo::addHook() {
+        static bool done = false;
+        if (!done) {
+            LOG(1) << "adding sharding hook" << endl;
+            pool.addHook(new ShardingConnectionHook(false));
+            shardConnectionPool.addHook(new ShardingConnectionHook(true));
+            done = true;
+        }
+    }
+
+    void ShardedConnectionInfo::setID( const OID& id ) {
         _id = id;
     }
 
     // -----ShardedConnectionInfo END ----
 
-    unsigned long long extractVersion( BSONElement e , string& errmsg ){
-        if ( e.eoo() ){
+    unsigned long long extractVersion( BSONElement e , string& errmsg ) {
+        if ( e.eoo() ) {
             errmsg = "no version";
             return 0;
         }
-        
+
         if ( e.isNumber() )
             return (unsigned long long)e.number();
-        
+
         if ( e.type() == Date || e.type() == Timestamp )
             return e._numberLong();
 
-        
+
         errmsg = "version is not a numeric type";
         return 0;
     }
 
     class MongodShardCommand : public Command {
     public:
-        MongodShardCommand( const char * n ) : Command( n ){
+        MongodShardCommand( const char * n ) : Command( n ) {
         }
         virtual bool slaveOk() const {
             return false;
@@ -333,12 +379,12 @@ namespace mongo {
             return true;
         }
     };
-    
-    
-    bool haveLocalShardingInfo( const string& ns ){
+
+
+    bool haveLocalShardingInfo( const string& ns ) {
         if ( ! shardingState.enabled() )
             return false;
-        
+
         if ( ! shardingState.hasVersion( ns ) )
             return false;
 
@@ -347,117 +393,192 @@ namespace mongo {
 
     class UnsetShardingCommand : public MongodShardCommand {
     public:
-        UnsetShardingCommand() : MongodShardCommand("unsetSharding"){}
+        UnsetShardingCommand() : MongodShardCommand("unsetSharding") {}
 
         virtual void help( stringstream& help ) const {
             help << " example: { unsetSharding : 1 } ";
         }
-        
-        virtual LockType locktype() const { return NONE; } 
- 
-        bool run(const string& , BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
+
+        virtual LockType locktype() const { return NONE; }
+
+        virtual bool slaveOk() const { return true; }
+
+        bool run(const string& , BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool) {
             ShardedConnectionInfo::reset();
             return true;
-        } 
-    
+        }
+
     } unsetShardingCommand;
-    
+
     class SetShardVersion : public MongodShardCommand {
     public:
-        SetShardVersion() : MongodShardCommand("setShardVersion"){}
+        SetShardVersion() : MongodShardCommand("setShardVersion") {}
 
         virtual void help( stringstream& help ) const {
             help << " example: { setShardVersion : 'alleyinsider.foo' , version : 1 , configdb : '' } ";
         }
+
+        virtual bool slaveOk() const { return true; }
+        virtual LockType locktype() const { return NONE; }
         
-        virtual LockType locktype() const { return WRITE; } // TODO: figure out how to make this not need to lock
- 
-        bool run(const string& , BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
+        bool checkConfigOrInit( const string& configdb , bool authoritative , string& errmsg , BSONObjBuilder& result , bool locked=false ) const {
+            if ( configdb.size() == 0 ) {
+                errmsg = "no configdb";
+                return false;
+            }
+            
+            if ( shardingState.enabled() ) {
+                if ( configdb == shardingState.getConfigServer() ) 
+                    return true;
+                
+                result.append( "configdb" , BSON( "stored" << shardingState.getConfigServer() << 
+                                                  "given" << configdb ) );
+                errmsg = "specified a different configdb!";
+                return false;
+            }
+            
+            if ( ! authoritative ) {
+                result.appendBool( "need_authoritative" , true );
+                errmsg = "first setShardVersion";
+                return false;
+            }
+            
+            if ( locked ) {
+                ShardedConnectionInfo::addHook();
+                shardingState.enable( configdb );
+                configServer.init( configdb );
+                return true;
+            }
+
+            dblock lk;
+            return checkConfigOrInit( configdb , authoritative , errmsg , result , true );
+        }
+        
+        bool checkMongosID( ShardedConnectionInfo* info, const BSONElement& id, string& errmsg ) {
+            if ( id.type() != jstOID ) {
+                if ( ! info->hasID() ) {
+                    warning() << "bad serverID set in setShardVersion and none in info: " << id << endl;
+                }
+                // TODO: fix this
+                //errmsg = "need serverID to be an OID";
+                //return 0;
+                return true;
+            }
+            
+            OID clientId = id.__oid();
+            if ( ! info->hasID() ) {
+                info->setID( clientId );
+                return true;
+            }
+            
+            if ( clientId != info->getID() ) {
+                errmsg = "server id has changed!";
+                return false;
+            }
+
+            return true;
+        }
+
+        bool run(const string& , BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool) {
+
+            // Steps
+            // 1. check basic config
+            // 2. extract params from command
+            // 3. fast check
+            // 4. slow check (LOCKS)
+            
+            // step 1
 
             lastError.disableForCommand();
             ShardedConnectionInfo* info = ShardedConnectionInfo::get( true );
 
+            // make sure we have the mongos id for writebacks
+            if ( ! checkMongosID( info , cmdObj["serverID"] , errmsg ) ) 
+                return false;
+
             bool authoritative = cmdObj.getBoolField( "authoritative" );
+            
+            // check config server is ok or enable sharding
+            if ( ! checkConfigOrInit( cmdObj["configdb"].valuestrsafe() , authoritative , errmsg , result ) )
+                return false;
 
-            string configdb = cmdObj["configdb"].valuestrsafe();
-            { // configdb checking
-                if ( configdb.size() == 0 ){
-                    errmsg = "no configdb";
-                    return false;
-                }
-                
-                if ( shardingState.enabled() ){
-                    if ( configdb != shardingState.getConfigServer() ){
-                        errmsg = "specified a different configdb!";
-                        return false;
-                    }
-                }
-                else {
-                    if ( ! authoritative ){
-                        result.appendBool( "need_authoritative" , true );
-                        errmsg = "first setShardVersion";
-                        return false;
-                    }
-                    shardingState.enable( configdb );
-                    configServer.init( configdb );
-                }
-            }
-
-            if ( cmdObj["shard"].type() == String ){
+            // check shard name/hosts are correct
+            if ( cmdObj["shard"].type() == String ) {
                 shardingState.gotShardName( cmdObj["shard"].String() );
                 shardingState.gotShardHost( cmdObj["shardHost"].String() );
             }
+            
 
-            { // setting up ids
-                if ( cmdObj["serverID"].type() != jstOID ){
-                    // TODO: fix this
-                    //errmsg = "need serverID to be an OID";
-                    //return 0;
-                }
-                else {
-                    OID clientId = cmdObj["serverID"].__oid();
-                    if ( ! info->hasID() ){
-                        info->setID( clientId );
-                    }
-                    else if ( clientId != info->getID() ){
-                        errmsg = "server id has changed!";
-                        return 0;
-                    }
-                }
+            // Handle initial shard connection
+            if( cmdObj["version"].eoo() && cmdObj["init"].trueValue() ){
+                result.append( "initialized", true );
+                return true;
             }
 
-            unsigned long long version = extractVersion( cmdObj["version"] , errmsg );
-
-            if ( errmsg.size() ){
+            // we can run on a slave up to here
+            if ( ! isMaster( "admin" ) ) {
+                result.append( "errmsg" , "not master" );
+                result.append( "note" , "from post init in setShardVersion" );
                 return false;
             }
+
+            // step 2
             
             string ns = cmdObj["setShardVersion"].valuestrsafe();
-            if ( ns.size() == 0 ){
-                errmsg = "need to speciy fully namespace";
+            if ( ns.size() == 0 ) {
+                errmsg = "need to specify namespace";
                 return false;
             }
+
+            const ConfigVersion version = extractVersion( cmdObj["version"] , errmsg );
+            if ( errmsg.size() )
+                return false;
             
+            // step 3
+
             const ConfigVersion oldVersion = info->getVersion(ns);
             const ConfigVersion globalVersion = shardingState.getVersion(ns);
+
+            result.appendTimestamp( "oldVersion" , oldVersion );
             
-            if ( oldVersion > 0 && globalVersion == 0 ){
+            if ( globalVersion > 0 && version > 0 ) {
+                // this means there is no reset going on an either side
+                // so its safe to make some assumptions
+
+                if ( version == globalVersion ) {
+                    // mongos and mongod agree!
+                    if ( oldVersion != version ) {
+                        assert( oldVersion < globalVersion );
+                        info->setVersion( ns , version );
+                    }
+                    return true;
+                }
+                
+            }
+
+            // step 4
+            
+            // this is because of a weird segfault I saw and I can't see why this should ever be set
+            massert( 13647 , str::stream() << "context should be empty here, is: " << cc().getContext()->ns() , cc().getContext() == 0 ); 
+        
+            dblock setShardVersionLock; // TODO: can we get rid of this??
+            
+            if ( oldVersion > 0 && globalVersion == 0 ) {
                 // this had been reset
                 info->setVersion( ns , 0 );
             }
 
-            if ( version == 0 && globalVersion == 0 ){
+            if ( version == 0 && globalVersion == 0 ) {
                 // this connection is cleaning itself
                 info->setVersion( ns , 0 );
                 return true;
             }
 
-            if ( version == 0 && globalVersion > 0 ){
-                if ( ! authoritative ){
+            if ( version == 0 && globalVersion > 0 ) {
+                if ( ! authoritative ) {
                     result.appendBool( "need_authoritative" , true );
                     result.append( "ns" , ns );
                     result.appendTimestamp( "globalVersion" , globalVersion );
-                    result.appendTimestamp( "oldVersion" , oldVersion );
                     errmsg = "dropping needs to be authoritative";
                     return false;
                 }
@@ -470,29 +591,29 @@ namespace mongo {
                 return true;
             }
 
-            if ( version < oldVersion ){
-                errmsg = "you already have a newer version of collection '" + ns + "'";
+            if ( version < oldVersion ) {
+                errmsg = "this connection already had a newer version of collection '" + ns + "'";
                 result.append( "ns" , ns );
-                result.appendTimestamp( "oldVersion" , oldVersion );
                 result.appendTimestamp( "newVersion" , version );
                 result.appendTimestamp( "globalVersion" , globalVersion );
                 return false;
             }
-            
-            if ( version < globalVersion ){
-                while ( shardingState.inCriticalMigrateSection() ){
+
+            if ( version < globalVersion ) {
+                while ( shardingState.inCriticalMigrateSection() ) {
                     dbtemprelease r;
                     sleepmillis(2);
-                    log() << "waiting till out of critical section" << endl;
+                    OCCASIONALLY log() << "waiting till out of critical section" << endl;
                 }
-                errmsg = "going to older version for global for collection '" + ns + "'";
+                errmsg = "shard global version for collection is higher than trying to set to '" + ns + "'";
                 result.append( "ns" , ns );
                 result.appendTimestamp( "version" , version );
                 result.appendTimestamp( "globalVersion" , globalVersion );
+                result.appendBool( "reloadConfig" , true );
                 return false;
             }
-            
-            if ( globalVersion == 0 && ! cmdObj.getBoolField( "authoritative" ) ){
+
+            if ( globalVersion == 0 && ! authoritative ) {
                 // need authoritative for first look
                 result.append( "ns" , ns );
                 result.appendBool( "need_authoritative" , true );
@@ -500,91 +621,98 @@ namespace mongo {
                 return false;
             }
 
+            Timer relockTime;
             {
                 dbtemprelease unlock;
 
                 ShardChunkVersion currVersion = version;
-                if ( ! shardingState.trySetVersion( ns , currVersion ) ){
-                    errmsg = str::stream() << "client version differs from config's for colleciton '" << ns << "'";
+                if ( ! shardingState.trySetVersion( ns , currVersion ) ) {
+                    errmsg = str::stream() << "client version differs from config's for collection '" << ns << "'";
                     result.append( "ns" , ns );
                     result.appendTimestamp( "version" , version );
                     result.appendTimestamp( "globalVersion" , currVersion );
                     return false;
                 }
             }
-
+            if ( relockTime.millis() >= ( cmdLine.slowMS - 10 ) ) {
+                log() << "setShardVersion - relocking slow: " << relockTime.millis() << endl;
+            }
+            
             info->setVersion( ns , version );
-            result.appendTimestamp( "oldVersion" , oldVersion );
-            result.append( "ok" , 1 );
-
             return true;
         }
-        
+
     } setShardVersionCmd;
-    
+
     class GetShardVersion : public MongodShardCommand {
     public:
-        GetShardVersion() : MongodShardCommand("getShardVersion"){}
+        GetShardVersion() : MongodShardCommand("getShardVersion") {}
 
         virtual void help( stringstream& help ) const {
             help << " example: { getShardVersion : 'alleyinsider.foo'  } ";
         }
-        
-        virtual LockType locktype() const { return NONE; } 
 
-        bool run(const string& , BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){
+        virtual LockType locktype() const { return NONE; }
+
+        bool run(const string& , BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool) {
             string ns = cmdObj["getShardVersion"].valuestrsafe();
-            if ( ns.size() == 0 ){
-                errmsg = "need to speciy fully namespace";
+            if ( ns.size() == 0 ) {
+                errmsg = "need to specify full namespace";
                 return false;
             }
-            
+
             result.append( "configServer" , shardingState.getConfigServer() );
 
             result.appendTimestamp( "global" , shardingState.getVersion(ns) );
-            
+
             ShardedConnectionInfo* info = ShardedConnectionInfo::get( false );
+            result.appendBool( "inShardedMode" , info != 0 );
             if ( info )
                 result.appendTimestamp( "mine" , info->getVersion(ns) );
-            else 
+            else
                 result.appendTimestamp( "mine" , 0 );
-            
+
             return true;
         }
-        
+
     } getShardVersion;
 
     class ShardingStateCmd : public MongodShardCommand {
     public:
-        ShardingStateCmd() : MongodShardCommand( "shardingState" ){}
+        ShardingStateCmd() : MongodShardCommand( "shardingState" ) {}
 
         virtual LockType locktype() const { return WRITE; } // TODO: figure out how to make this not need to lock
 
-        bool run(const string& , BSONObj& cmdObj, string& errmsg, BSONObjBuilder& result, bool){        
+        bool run(const string& , BSONObj& cmdObj, int, string& errmsg, BSONObjBuilder& result, bool) {
             shardingState.appendInfo( result );
             return true;
         }
-        
+
     } shardingStateCmd;
 
     /**
      * @ return true if not in sharded mode
                      or if version for this client is ok
      */
-    bool shardVersionOk( const string& ns , bool isWriteOp , string& errmsg ){
+    bool shardVersionOk( const string& ns , string& errmsg ) {
         if ( ! shardingState.enabled() )
             return true;
 
+        if ( ! isMasterNs( ns.c_str() ) )  {
+            // right now connections to secondaries aren't versioned at all
+            return true;
+        }
+
         ShardedConnectionInfo* info = ShardedConnectionInfo::get( false );
 
-        if ( ! info ){
+        if ( ! info ) {
             // this means the client has nothing sharded
             // so this allows direct connections to do whatever they want
             // which i think is the correct behavior
             return true;
         }
-        
-        if ( info->inForceVersionOkMode() ){
+
+        if ( info->inForceVersionOkMode() ) {
             return true;
         }
 
@@ -592,35 +720,35 @@ namespace mongo {
         //   all collections at some point, be sharded or not, will have a version (and a ShardChunkManager)
         //   for now, we remove the sharding state of dropped collection
         //   so delayed request may come in. This has to be fixed.
-        ConfigVersion version;    
-        if ( ! shardingState.hasVersion( ns , version ) ){
+        ConfigVersion clientVersion = info->getVersion(ns);
+        ConfigVersion version;
+        if ( ! shardingState.hasVersion( ns , version ) && clientVersion == 0 ) {
             return true;
         }
 
-        ConfigVersion clientVersion = info->getVersion(ns);
 
-        if ( version == 0 && clientVersion > 0 ){
+        if ( version == 0 && clientVersion > 0 ) {
             stringstream ss;
-            ss << "collection was dropped or this shard no longer valied version: " << version << " clientVersion: " << clientVersion;
+            ss << "collection was dropped or this shard no longer valid version: " << version << " clientVersion: " << clientVersion;
             errmsg = ss.str();
             return false;
         }
-        
+
         if ( clientVersion >= version )
             return true;
-        
 
-        if ( clientVersion == 0 ){
+
+        if ( clientVersion == 0 ) {
             stringstream ss;
             ss << "client in sharded mode, but doesn't have version set for this collection: " << ns << " myVersion: " << version;
             errmsg = ss.str();
             return false;
         }
 
-        if ( isWriteOp && version.majorVersion() == clientVersion.majorVersion() ){
-            // this means there was just a split 
+        if ( version.majorVersion() == clientVersion.majorVersion() ) {
+            // this means there was just a split
             // since on a split w/o a migrate this server is ok
-            // going to accept write
+            // going to accept 
             return true;
         }
 
@@ -630,4 +758,7 @@ namespace mongo {
         return false;
     }
 
+    void ShardingConnectionHook::onHandedOut( DBClientBase * conn ) {
+        // no-op for mongod
+    }
 }
