@@ -1000,8 +1000,8 @@ jsTestOptions = function(){
     return {}
 }
 
-testLog = function(x){
-    print( jsTestFile() + " - " + x )
+jsTestLog = function(msg){
+    print( "\n\n----\n" + msg + "\n----\n\n" )
 }
 
 shellPrintHelper = function (x) {
@@ -1146,7 +1146,7 @@ shellAutocomplete.showPrivate = false; // toggle to show (useful when working on
 
 shellHelper = function( command , rest , shouldPrint ){
     command = command.trim();
-    var args = rest.trim().replace(/\s+;$/,"").split( "\s+" );
+    var args = rest.trim().replace(/\s*;$/,"").split( "\s+" );
     
     if ( ! shellHelper[command] )
         throw "no command [" + command + "]";
